SystemOrganization addCategory: #'Magritte-Json'!

WAObject subclass: #MJObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Json'!

!MJObject commentStamp: 'pmm 9/2/2009 14:46' prior: 0!
A MJObject is a common base class that provides some convenience methods for JSON/Smalltalk mapping.
!

!MJObject class methodsFor: 'accessing-description' stamp: 'pmm 9/1/2009 17:22'!
descriptionContainer
	"Return the default description container."

	^ MAPriorityContainer new
		label: self label;
		kind: self;
		yourself! !

!MJObject class methodsFor: 'instance creation' stamp: 'pmm 9/1/2009 09:36'!
fromJavascript: aString
	^MJJsonParser parse: aString using: self description! !

!MJObject methodsFor: 'mapping' stamp: 'pmm 8/31/2009 20:48'!
javascriptOn: aStream
	| description |
	description := self description.
	description jsonWriter
		write: self
		description: description
		to: aStream! !

MAWriter subclass: #MJJsonWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Json'!

!MJJsonWriter methodsFor: 'private' stamp: 'pmm 8/31/2009 09:29'!
defaultWriteStream
	^ WriteStream on: String new! !

!MJJsonWriter methodsFor: 'visiting-description' stamp: 'pmm 9/2/2009 16:57'!
visitContainer: aDescription
	| first |
	first := true.
	stream nextPut: ${.
	aDescription do: [ :each |
		each isJsonSerialized ifTrue: [
			| nextObject |
			nextObject := self object readUsing: each.
			nextObject isNil ifFalse: [
				first
					ifTrue: [ first := false ]
					ifFalse: [ stream nextPut: $, ].
				self writeElementKeyOf: each.
				stream nextPut: $:.
				each jsonWriter
					write: nextObject
					description: each to: stream ] ] ].
	stream nextPut: $}! !

!MJJsonWriter methodsFor: 'visiting-description' stamp: 'pmm 8/31/2009 23:27'!
visitDateAndTimeDescription: aDescription
	self object printString javascriptOn: self stream! !

!MJJsonWriter methodsFor: 'visiting-description' stamp: 'pmm 8/31/2009 20:42'!
visitElementDescription: aDescription
	self object javascriptOn: self stream! !

!MJJsonWriter methodsFor: 'private' stamp: 'pmm 8/31/2009 20:48'!
writeElementKeyOf: aDescription
	JSJsonStream
		encodeKey: aDescription jsonName
		on: self stream.! !

JSJsonParser subclass: #MJJsonParser
	instanceVariableNames: 'description'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Json'!

!MJJsonParser class methodsFor: 'instance creation' stamp: 'pmm 9/1/2009 09:34'!
onStream: aStream description: aDescription
	^ self basicNew initializeOnStream: aStream description: aDescription! !

!MJJsonParser class methodsFor: 'instance creation' stamp: 'pmm 9/1/2009 09:39'!
parse: aString using: aDescription
	^(self onStream: aString readStream description: aDescription) parse! !

!MJJsonParser methodsFor: 'adding' stamp: 'pmm 9/1/2009 11:00'!
addProperty: anAssociation to: anObject
	| child value |
	child := anObject description detect: [ :each |
		each jsonName = anAssociation key ].
	value := anAssociation value.
	value isString ifTrue: [
		value := child jsonReader read: value readStream description: child ].
	anObject write: value using: child.
	^ anObject! !

!MJJsonParser methodsFor: 'creating' stamp: 'pmm 9/1/2009 10:49'!
createObject
	^description kind new! !

!MJJsonParser methodsFor: 'initialize-release' stamp: 'pmm 9/1/2009 09:32'!
initializeOnStream: aStream description: aDescription
	super initializeOn: aStream.
	description := aDescription! !

!MJJsonParser methodsFor: 'accessing' stamp: 'pmm 9/1/2009 09:26'!
pushDescription: aDescription during: aBlock
	| old |
	old := self description.
	description := aDescription.
	aBlock value.
	description := old.! !

MAStringReader subclass: #MJJsonReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Json'!

!MJJsonReader methodsFor: 'visiting-description' stamp: 'pmm 9/1/2009 11:00'!
visitDateAndTimeDescription: aDescription
	self object: (DateAndTime readFrom: self stream)! !

!MADescription class methodsFor: '*Magritte-Json' stamp: 'pmm 8/31/2009 09:34'!
defaultJsonReader
	^ MJJsonReader! !

!MADescription class methodsFor: '*Magritte-Json' stamp: 'pmm 9/3/2009 11:47'!
defaultJsonSerialized
	^self defaultVisible! !

!MADescription class methodsFor: '*Magritte-Json' stamp: 'pmm 8/31/2009 09:35'!
defaultJsonWriter
	^ MJJsonWriter! !

!MADescription class methodsFor: '*Magritte-Json' stamp: 'pmm 9/3/2009 11:50'!
descriptionJsonReader
	^ MASingleOptionDescription new
		accessor: #jsonReader;
		label: 'JSON Reader';
		priority: 320;
		default: self defaultJsonReader;
		options: [ self defaultJsonReader withAllSubclasses ] asDynamicObject;
		reference: MAClassDescription new;
		yourself! !

!MADescription class methodsFor: '*Magritte-Json' stamp: 'pmm 9/3/2009 11:51'!
descriptionStringWriter
	^ MASingleOptionDescription new
		accessor: #jsonWriter;
		label: 'JSON Writer';
		priority: 330;
		default: self defaultJsonWriter;
		options: [ self defaultJsonWriter withAllSubclasses ] asDynamicObject;
		reference: MAClassDescription new;
		yourself! !

!MADescription class methodsFor: '*Magritte-Json' stamp: 'pmm 9/3/2009 11:47'!
descriptionVisible
	^ MABooleanDescription new
		accessor: #isJsonSerialized;
		label: 'JSON Serialized';
		priority: 220;
		default: self defaultJsonSerialized;
		yourself! !

!MADescription methodsFor: '*Magritte-Json' stamp: 'pmm 9/3/2009 11:46'!
isJsonSerialized
	"Answer whether the described object should be serialized to JSON."

	^ self propertyAt: #jsonSerialized ifAbsent: [ self class defaultJsonSerialized ]! !

!MADescription methodsFor: '*Magritte-Json' stamp: 'pmm 9/3/2009 11:45'!
isJsonSerialized: aBoolean

	^ self propertyAt: #jsonSerialized put: aBoolean! !

!MADescription methodsFor: '*Magritte-Json' stamp: 'pmm 8/31/2009 09:36'!
jsonReader
	"Answer a Visitor that can be used to parse the model described by the receiver from a JSON string."

	^ self propertyAt: #jsonReader ifAbsent: [ self class defaultJsonReader ]! !

!MADescription methodsFor: '*Magritte-Json' stamp: 'pmm 8/31/2009 09:36'!
jsonReader: aClass
	self propertyAt: #jsonReader put: aClass! !

!MADescription methodsFor: '*Magritte-Json' stamp: 'pmm 8/31/2009 09:37'!
jsonWriter
	"Answer a Visitor that can be used to convert the model described by the receiver to a JSON string."

	^ self propertyAt: #jsonWriter ifAbsent: [ self class defaultJsonWriter ]! !

!MADescription methodsFor: '*Magritte-Json' stamp: 'pmm 8/31/2009 09:37'!
jsonWriter: aClass
	"Answer a Visitor that can be used to convert the model described by the receiver to a JSON string."

	self propertyAt: #jsonWriter put: aClass! !

!MAElementDescription methodsFor: '*Magritte-Json' stamp: 'pmm 8/31/2009 19:53'!
defaultJsonName
	"Answer the default JSON element name."

	^ self accessor selector seasideString capitalized! !

!MAElementDescription methodsFor: '*Magritte-Json' stamp: 'pmm 8/31/2009 19:53'!
jsonName
	"Answer the JSON element name."

	^ self propertyAt: #jsonName ifAbsent: [ self defaultJsonName ]! !

!MAElementDescription methodsFor: '*Magritte-Json' stamp: 'pmm 8/31/2009 19:52'!
jsonName: aString
	self propertyAt: #jsonName put: aString! !
